@@ -5,6 +5,7 @@
 #'
 #' @export
 #' @export as.data.table
+#' @family output
 #' @aliases as.data.table
 #' @importFrom data.table as.data.table
 #' @param keep.rownames \[`logical(1)`]\cr Not used.
@@ -86,10 +87,11 @@ as.data.table.dynamitefit <- function(x, keep.rownames = FALSE,
   if (!is.null(parameters)) {
     responses <- types <- NULL
   }
+  all_responses <- unique(c(names(x$stan$responses), unlist(x$stan$responses)))
   if (is.null(responses)) {
-    responses <- setdiff(unique(x$priors$response), "")
+    responses <- all_responses
   } else {
-    z <- responses %in% unique(x$priors$response)
+    z <- responses %in% all_responses
     stopifnot_(
       all(z),
       "Model does not contain response variable{?s} {.var {responses[!z]}}."
@@ -97,11 +99,16 @@ as.data.table.dynamitefit <- function(x, keep.rownames = FALSE,
   }
   all_types <- c(
     "alpha", "beta", "delta", "tau", "tau_alpha", "xi",
-    "sigma_nu", "corr_nu", "sigma", "phi", "nu", "lambda", "sigma_lambda",
-    "psi", "tau_psi", "corr_psi", "omega", "omega_alpha", "omega_psi"
+    "sigma_nu", "sigma", "phi", "nu", "lambda", "sigma_lambda",
+    "psi", "tau_psi", "corr", "corr_psi", "corr_nu",
+    "omega", "omega_alpha", "omega_psi"
   )
   if (is.null(types)) {
-    types <- all_types[seq_len(16L)]
+    types <- ifelse_(
+      is.null(parameters),
+      all_types[!grepl("omega", all_types, fixed = TRUE)],
+      all_types
+    )
   } else {
     types <- onlyif(is.character(types), tolower(types))
     types <- try(match.arg(types, all_types, TRUE), silent = TRUE)
@@ -110,7 +117,12 @@ as.data.table.dynamitefit <- function(x, keep.rownames = FALSE,
       "Argument {.arg type} contains unknown types."
     )
   }
-  values <- function(type, response) {
+  values <- function(type, response, category) {
+    ycat <- ifelse_(
+      nzchar(category) & !is.na(category),
+      paste0("_", category),
+      ""
+    )
     if (type %in% c("xi", "corr_nu", "corr_psi")) {
       draws <- rstan::extract(
         x$stanfit,
@@ -120,14 +132,17 @@ as.data.table.dynamitefit <- function(x, keep.rownames = FALSE,
     } else {
       draws <- rstan::extract(
         x$stanfit,
-        pars = paste0(type, "_", response),
+        pars = paste0(type, "_", response, ycat),
         permuted = FALSE
       )
     }
-    category <- attr(x$stan$responses[[response]], "levels")[-1L]
-    if (is.null(category)) {
-      category <- NA
-    }
+    channel <- get_channel(x, response)
+    idx <- which(names(x$stan$responses) %in% response)
+    resps <- ifelse_(
+      identical(length(idx), 0L),
+      NULL,
+      x$stan$responses[[idx]]
+    )
     d <- do.call(
       what = paste0("as_data_table_", type),
       args = list(
@@ -136,7 +151,8 @@ as.data.table.dynamitefit <- function(x, keep.rownames = FALSE,
         n_draws = prod(dim(draws)[1L:2L]),
         response = response,
         category = category,
-        include_fixed = include_fixed
+        include_fixed = include_fixed,
+        resps = resps
       )
     )
     n_d <- d[, .N]
@@ -151,12 +167,14 @@ as.data.table.dynamitefit <- function(x, keep.rownames = FALSE,
   }
   # avoid NSE notes from R CMD check
   .chain <- .draw <- .iteration <- NULL
-  category <- group <- parameter <- response <- time <- type <- value <- NULL
+  category <- group <- parameter <- response <- NULL
+  catstr <- time <- type <- value <- NULL
   out_all <- NULL
   if ("xi" %in% types) {
     out_all <- data.table::data.table(
       type = "xi",
       response = "",
+      category = NA_character_,
       parameter = "xi"
     )
   }
@@ -166,6 +184,7 @@ as.data.table.dynamitefit <- function(x, keep.rownames = FALSE,
       data.table::data.table(
         type = "corr_nu",
         response = "",
+        category = NA_character_,
         parameter = "corr_nu"
       )
     )
@@ -176,37 +195,50 @@ as.data.table.dynamitefit <- function(x, keep.rownames = FALSE,
       data.table::data.table(
         type = "corr_psi",
         response = "",
+        category = NA_character_,
         parameter = "corr_psi"
       )
     )
   }
+  categories <- c(
+    NA_character_,
+    ulapply(
+      attr(x$stan$responses, "resp_class"),
+      function(y) attr(y, "levels")[-1L]
+    )
+  )
   tmp <- data.table::as.data.table(
     expand.grid(
       type = types,
       response = responses,
+      category = categories,
       stringsAsFactors = FALSE
     )
   )
-  tmp[, parameter := as.character(glue::glue("{tmp$type}_{tmp$response}"))]
+  tmp[, catstr := ifelse(
+    nzchar(category) & !is.na(category),
+    glue::glue("_{tmp$category}"),
+    ""
+  )]
+  tmp[, parameter := as.character(
+    glue::glue("{tmp$type}_{tmp$response}{tmp$catstr}")
+  )]
+  tmp[, catstr := NULL]
   out <- data.table::rbindlist(list(out_all, tmp))
   rows <- apply(out, 1L, function(y) {
     any(
       grepl(
-        paste0("^", y["parameter"]),
+        paste0("^", y["parameter"], "$"),
         x$stanfit@sim$pars_oi
       )
     )
   })
-  out <- out[rows, c("response", "type")]
+  out <- out[rows, c("response", "category", "type")]
   n_pars <- nrow(out)
   stopifnot_(
     n_pars > 0L,
-    paste0(
-      "No parameters of type {.var ",
-      paste(types, collapse = "}, {.var "),
-      "} found for any of the response channels {.var ",
-      paste(responses, collapse = "}, {.var "), "}."
-    )
+    "No parameters of type {.var {types}} found for any of the response
+     channels {.var {responses}}."
   )
   all_values <- vector(mode = "list", length = n_pars + 1L)
   # template for rbindlist
@@ -224,26 +256,30 @@ as.data.table.dynamitefit <- function(x, keep.rownames = FALSE,
   )
   all_values[seq.int(2L, n_pars + 1L)] <- .mapply(
     values,
-    dots = list(type = out$type, response = out$response),
+    dots = list(
+      type = out$type,
+      response = out$response,
+      category = out$category
+    ),
     MoreArgs = NULL
   )
   out <- data.table::rbindlist(all_values, fill = TRUE)
   if (!is.null(parameters)) {
     data.table::setkey(out, "parameter")
     found_pars <- parameters %in% unique(out$parameter)
-    stopifnot_(all(found_pars),
+    stopifnot_(
+      all(found_pars),
       c(
-      "Parameter{?s} {.var {parameters[!found_pars]}} not found in
-        the model output.",
-        `x` = "Use {.fun get_parameter_names} to check available parameters."
+        "Parameter{?s} {.var {parameters[!found_pars]}} not found in
+         the model output.",
+        `i` = "Use {.fun get_parameter_names} to check available parameters."
       )
     )
     out <- out[parameters]
   }
   if (summary) {
     pars <- unique(out$parameter)
-    out <- out[
-      ,
+    out <- out[,
       parameter := factor(parameter, levels = pars, ordered = TRUE)
     ][,
       {
@@ -254,8 +290,7 @@ as.data.table.dynamitefit <- function(x, keep.rownames = FALSE,
         c(list(mean = mean, sd = sd), tmp)
       },
       by = list(parameter, time, group, category, response, type)
-    ][
-      ,
+    ][,
       parameter := as.character(parameter)
     ]
     pnames <- c("time", "group", "category", "response", "type")
@@ -265,6 +300,19 @@ as.data.table.dynamitefit <- function(x, keep.rownames = FALSE,
   out
 }
 
+#' Get Channel Variables or Channel Group Variables
+#'
+#' @param x A `dynamitefit` object.
+#' @param response The response variable name.
+#' @noRd
+get_channel <- function(x, response) {
+  if (is.null(x$stan$channel_vars[[response]])) {
+    x$stan$channel_group_vars[[response]]
+  } else {
+    x$stan$channel_vars[[response]]
+  }
+}
+
 #' Construct a Data Table for a Parameter Type from a `dynamitefit` Object
 #'
 #' Arguments for all as_data_frame_type functions are documented here.
@@ -275,7 +323,7 @@ as.data.table.dynamitefit <- function(x, keep.rownames = FALSE,
 #' @param response \[`character(1)`]\cr Response variable name.
 #' @param categories \[`character()`]\cr Levels of categorical responses.
 #' @noRd
-as_data_table_default <- function(type, draws, response) {
+as_data_table_default <- function(type, draws, response, ...) {
   data.table::data.table(
     parameter = paste0(type, "_", response),
     value = c(draws)
@@ -284,9 +332,13 @@ as_data_table_default <- function(type, draws, response) {
 
 #' @describeIn as_data_table_default Data Table for a "xi" Parameter
 #' @noRd
-as_data_table_xi <- function(x, draws, ...) {
+as_data_table_xi <- function(x, draws, n_draws, ...) {
+  D <- x$stan$model_vars$D
   data.table::data.table(
-    parameter = "xi",
+    parameter = rep(
+      paste0("xi_d", seq_len(D - 1L)),
+      each = n_draws
+    ),
     value = c(draws)
   )
 }
@@ -294,15 +346,28 @@ as_data_table_xi <- function(x, draws, ...) {
 #' @describeIn as_data_table_default Data Table for a "corr_nu" Parameter
 #' @noRd
 as_data_table_corr_nu <- function(x, draws, n_draws, ...) {
-  vars <- unlist(lapply(x$stan$model_vars, function(x) {
-    icpt <- ifelse_(
-      x$has_random_intercept,
-      "alpha",
-      NULL
-    )
-    paste0(x$resp, "_", c(icpt, names(x$J_random)))
-  }))
-
+  vars <- ulapply(
+    x$stan$channel_vars,
+    function(y) {
+      if (y$has_random || y$has_random_intercept) {
+        icpt <- ifelse_(
+          y$has_random_intercept,
+          "alpha",
+          NULL
+        )
+        vars <- paste0(y$y, "_", c(icpt, names(y$J_random)))
+        ifelse_(
+          is_categorical(y$family),
+          paste0(
+            rep(vars, y$S - 1L),
+            "_",
+            rep(y$categories[-1L], each = y$K_random)
+          ),
+          vars
+        )
+      }
+    }
+  )
   pairs <- apply(utils::combn(vars, 2L), 2L, paste, collapse = "__")
   data.table::data.table(
     parameter = rep(paste0("corr_nu_", pairs), each = n_draws),
@@ -312,34 +377,34 @@ as_data_table_corr_nu <- function(x, draws, n_draws, ...) {
 
 #' @describeIn as_data_table_default Data Table for a "nu" Parameter
 #' @noRd
-as_data_table_nu <- function(x, draws, n_draws, response, ...) {
+as_data_table_nu <- function(x, draws, n_draws, response, category, ...) {
   icpt <- ifelse_(
-    x$stan$model_vars[[response]]$has_random_intercept,
+    get_channel(x, response)$has_random_intercept,
     "alpha",
     NULL
   )
   var_names <- paste0(
     "nu_", response, "_",
-    c(icpt, names(x$stan$model_vars[[response]]$J_random))
+    c(icpt, names(get_channel(x, response)$J_random))
   )
   n_vars <- length(var_names)
   groups <- sort(unique(x$data[[x$group_var]]))
-  n_groups <- length(groups)
+  n_group <- length(groups)
   data.table::data.table(
-    parameter = rep(var_names, each = n_draws * n_groups),
+    parameter = rep(var_names, each = n_draws * n_group),
     value = c(draws),
-    group = rep(groups, each = n_draws)
+    group = rep(groups, each = n_draws),
+    category = category
   )
 }
 
 #' @describeIn as_data_table_default Data Table for a "alpha" Parameter
 #' @noRd
 as_data_table_alpha <- function(x, draws, n_draws,
-                                response, category, include_fixed) {
-  n_cat <- length(category)
+                                response, category, include_fixed, ...) {
   fixed <- x$stan$fixed
   all_time_points <- sort(unique(x$data[[x$time_var]]))
-  if (x$stan$model_vars[[response]]$has_varying_intercept) {
+  if (get_channel(x, response)$has_varying_intercept) {
     time_points <- ifelse_(
       include_fixed,
       all_time_points,
@@ -348,24 +413,20 @@ as_data_table_alpha <- function(x, draws, n_draws,
     n_na <- include_fixed * fixed * n_draws
     n_time <- length(time_points)
     n_time2 <- n_time - include_fixed * fixed
-    data.table::rbindlist(
-      lapply(seq_len(n_cat), function(i) {
-        data.table::data.table(
-          parameter = paste0("alpha_", response),
-          value = c(
-            rep(NA, n_na),
-            c(draws[, , (i - 1L) * n_time2 + seq_len(n_time2)])
-          ),
-          time = rep(time_points, each = n_draws),
-          category = category[i]
-        )
-      })
+    data.table::data.table(
+      parameter = paste0("alpha_", response),
+      value = c(
+        rep(NA, n_na),
+        c(draws[, , seq_len(n_time2)])
+      ),
+      time = rep(time_points, each = n_draws),
+      category = category
     )
   } else {
     data.table::data.table(
       parameter = paste0("alpha_", response),
       value = c(draws),
-      category = rep(category, each = n_draws)
+      category = category
     )
   }
 }
@@ -375,26 +436,25 @@ as_data_table_alpha <- function(x, draws, n_draws,
 as_data_table_beta <- function(x, draws, n_draws, response, category, ...) {
   var_names <- paste0(
     "beta_", response, "_",
-    names(x$stan$model_vars[[response]]$J_fixed)
+    names(get_channel(x, response)$J_fixed)
   )
   n_vars <- length(var_names)
   data.table::data.table(
     parameter = rep(var_names, each = n_draws),
     value = c(draws),
-    category = rep(category, each = n_vars * n_draws)
+    category = category
   )
 }
 
 #' @describeIn as_data_table_default Data Table for a "delta" Parameter
 #' @noRd
 as_data_table_delta <- function(x, draws, n_draws,
-                                response, category, include_fixed) {
-  n_cat <- length(category)
+                                response, category, include_fixed, ...) {
   fixed <- x$stan$fixed
   all_time_points <- sort(unique(x$data[[x$time_var]]))
   var_names <- paste0(
     "delta_", response, "_",
-    names(x$stan$model_vars[[response]]$J_varying)
+    names(get_channel(x, response)$J_varying)
   )
   n_vars <- length(var_names)
   time_points <- ifelse_(
@@ -405,19 +465,17 @@ as_data_table_delta <- function(x, draws, n_draws,
   n_na <- include_fixed * fixed * n_draws
   n_time <- length(time_points)
   n_time2 <- n_time - include_fixed * fixed
-  data.table::rbindlist(lapply(seq_len(n_cat), function(j) {
-    data.table::rbindlist(lapply(seq_len(n_vars), function(i) {
-      idx <- (j - 1L) * n_time2 * n_vars + (i - 1L) * n_time2 + seq_len(n_time2)
-      data.table::data.table(
-        parameter = var_names[i],
-        value = c(
-          rep(NA, n_na),
-          c(draws[, , idx])
-        ),
-        time = rep(time_points, each = n_draws),
-        category = rep(category[j], each = n_time * n_draws)
-      )
-    }))
+  data.table::rbindlist(lapply(seq_len(n_vars), function(i) {
+    idx <- (i - 1L) * n_time2 + seq_len(n_time2)
+    data.table::data.table(
+      parameter = var_names[i],
+      value = c(
+        rep(NA, n_na),
+        c(draws[, , idx])
+      ),
+      time = rep(time_points, each = n_draws),
+      category = category
+    )
   }))
 }
 
@@ -426,7 +484,7 @@ as_data_table_delta <- function(x, draws, n_draws,
 as_data_table_tau <- function(x, draws, n_draws, response, ...) {
   var_names <- paste0(
     "tau_", response, "_",
-    names(x$stan$model_vars[[response]]$J_varying)
+    names(get_channel(x, response)$J_varying)
   )
   data.table::data.table(
     parameter = rep(var_names, each = n_draws),
@@ -438,34 +496,34 @@ as_data_table_tau <- function(x, draws, n_draws, response, ...) {
 #' @noRd
 as_data_table_omega <- function(x, draws, n_draws, response, category, ...) {
   n_cat <- length(category)
-  D <- x$stan$sampling_vars$D
-  var_names <- names(x$stan$model_vars[[response]]$J_varying)
+  D <- x$stan$model_vars$D
+  var_names <- paste0(
+    "omega_", response, "_",
+    names(get_channel(x, response)$J_varying)
+  )
   k <- length(var_names)
   data.table::data.table(
     parameter = rep(
-      paste0(
-        "omega_", rep(seq_len(D), each = n_cat * k), "_",
-        rep(var_names, each = n_cat)
-      ),
+      paste0(var_names, "_d", rep(seq_len(D), each = k)),
       each = n_draws
     ),
     value = c(draws),
-    category = rep(category, each = n_draws)
+    category = category
   )
 }
 
 #' @describeIn as_data_table_default Data Table for a "omega_alpha" Parameter
 #' @noRd
-as_data_table_omega_alpha <- function(x, draws, n_draws, category, ...) {
-  n_cat <- length(category)
-  D <- x$stan$sampling_vars$D
+as_data_table_omega_alpha <-function(x, draws, n_draws, response,
+  category, ...) {
+  D <- x$stan$model_vars$D
   data.table::data.table(
     parameter = rep(
-      paste0("omega_alpha_", seq_len(D)),
-      each = n_cat * n_draws
+      paste0("omega_alpha_", response, "_d", seq_len(D)),
+      each = n_draws
     ),
     value = c(draws),
-    category = rep(category, each = n_draws)
+    category = category
   )
 }
 
@@ -483,19 +541,20 @@ as_data_table_sigma <- function(draws, response, ...) {
 
 #' @describeIn as_data_table_default Data Table for a "sigma_nu" Parameter
 #' @noRd
-as_data_table_sigma_nu <- function(x, draws, n_draws, response, ...) {
+as_data_table_sigma_nu <- function(x, draws, n_draws, response, category, ...) {
   icpt <- ifelse_(
-    x$stan$model_vars[[response]]$has_random_intercept,
+    get_channel(x, response)$has_random_intercept,
     "alpha",
     NULL
   )
   var_names <- paste0(
     "sigma_nu_", response, "_",
-    c(icpt, names(x$stan$model_vars[[response]]$J_random))
+    c(icpt, names(get_channel(x, response)$J_random))
   )
   data.table::data.table(
     parameter = rep(var_names, each = n_draws),
-    value = c(draws)
+    value = c(draws),
+    category = category
   )
 }
 
@@ -524,10 +583,8 @@ as_data_table_sigma_lambda <- function(draws, response, ...) {
 
 #' @describeIn as_data_table_default Data Table for a "psi" Parameter
 #' @noRd
-as_data_table_psi <- function(
-    x, draws, n_draws,
-    response, category, include_fixed) {
-  n_cat <- length(category)
+as_data_table_psi <- function(x, draws, n_draws, response,
+                              category, include_fixed, ...) {
   fixed <- x$stan$fixed
   all_time_points <- sort(unique(x$data[[x$time_var]]))
   time_points <- ifelse_(
@@ -538,18 +595,14 @@ as_data_table_psi <- function(
   n_na <- include_fixed * fixed * n_draws
   n_time <- length(time_points)
   n_time2 <- n_time - include_fixed * fixed
-  data.table::rbindlist(
-    lapply(seq_len(n_cat), function(i) {
-      data.table::data.table(
-        parameter = paste0("psi_", response),
-        value = c(
-          rep(NA, n_na),
-          c(draws[, , (i - 1L) * n_time2 + seq_len(n_time2)])
-        ),
-        time = rep(time_points, each = n_draws),
-        category = category[i]
-      )
-    })
+  data.table::data.table(
+    parameter = paste0("psi_", response),
+    value = c(
+      rep(NA, n_na),
+      c(draws[, , seq_len(n_time2)])
+    ),
+    time = rep(time_points, each = n_draws),
+    category = category
   )
 }
 
@@ -561,26 +614,36 @@ as_data_table_tau_psi <- function(draws, response, ...) {
 
 #' @describeIn as_data_table_default Data Table for a "omega_psi" Parameter
 #' @noRd
-as_data_table_omega_psi <- function(x, draws, n_draws, category, ...) {
-  n_cat <- length(category)
-  D <- x$stan$sampling_vars$D
+as_data_table_omega_psi <- function(x, draws, n_draws, response,
+                                    category, ...) {
+  D <- x$stan$model_vars$D
   data.table::data.table(
     parameter = rep(
-      paste0("omega_psi_", seq_len(D)),
-      each = n_cat * n_draws
+      paste0("omega_psi_", response, "_d", seq_len(D)),
+      each =  n_draws
     ),
     value = c(draws),
-    category = rep(category, each = n_draws)
+    category = category
   )
 }
 
 #' @describeIn as_data_table_default Data Table for a "corr_psi" Parameter
 #' @noRd
 as_data_table_corr_psi <- function(x, draws, n_draws, ...) {
-  resp <- attr(x$dformulas$stoch, "lfactor")
-  pairs <- apply(utils::combn(resp, 2L), 2L, paste, collapse = "_")
+  resp <- attr(x$dformulas$stoch, "lfactor")$responses
+  pairs <- apply(utils::combn(resp, 2L), 2L, paste, collapse = "__")
   data.table::data.table(
     parameter = rep(paste0("corr_psi_", pairs), each = n_draws),
     value = c(draws)
   )
 }
+
+#' @describeIn as_data_table_default Data Table for a "corr" Parameter
+#' @noRd
+as_data_table_corr <- function(x, draws, n_draws, resps, ...) {
+  pairs <- apply(utils::combn(resps, 2L), 2L, paste, collapse = "__")
+  data.table::data.table(
+    parameter = rep(paste0("corr_", pairs), each = n_draws),
+    value = c(draws)
+  )
+}
