@@ -30,13 +30,17 @@ test_data <- data.frame(
 
 debug <- list(no_compile = TRUE)
 
-test_that("single channel models are valid", {
+test_that("family-specific model formulas are valid", {
+  # Model formulas
   expect_error(
     obs_categorical <- obs(y1 ~ x1, family = "categorical"), NA
   )
   expect_error(
     obs_gaussian <- obs(y2 ~ x2, family = "gaussian"), NA
   )
+  expect_error(
+    obs_mvgaussian <- obs(c(y2, y8) ~ x2, family = "mvgaussian"), NA
+  )
   expect_error(
     obs_binomial <- obs(y3 ~ x3 + trials(trials), family = "binomial"), NA
   )
@@ -58,12 +62,26 @@ test_that("single channel models are valid", {
   expect_error(
     obs_beta <- obs(y9 ~ x3, family = "beta"), NA
   )
+  expect_error(
+    obs_student <- obs(y2 ~ x2, family = "student"), NA
+  )
+  expect_error(
+    obs_multinomial <- obs(
+      c(y3, y4, y5) ~ x2 + trials(trials),
+      family = "multinomial"
+    ),
+    NA
+  )
+  # Stan data
   expect_error(
     dynamite(obs_categorical, test_data, "time", "group", debug = debug), NA
   )
   expect_error(
     dynamite(obs_gaussian, test_data, "time", "group", debug = debug), NA
   )
+  expect_error(
+    dynamite(obs_mvgaussian, test_data, "time", "group", debug = debug), NA
+  )
   expect_error(
     dynamite(obs_binomial, test_data, "time", "group", debug = debug), NA
   )
@@ -85,6 +103,12 @@ test_that("single channel models are valid", {
   expect_error(
     dynamite(obs_beta, test_data, "time", "group", debug = debug), NA
   )
+  expect_error(
+    dynamite(obs_student, test_data, "time", "group", debug = debug), NA
+  )
+  expect_error(
+    dynamite(obs_multinomial, test_data, "time", "group", debug = debug), NA
+  )
 })
 
 test_that("multichannel models are valid", {
@@ -207,7 +231,7 @@ test_that("shrinkage is handled correctly", {
             "time",
             "group",
             debug = debug
-          )$stan$model_vars, "[[", "shrinkage"
+          )$stan$channel_vars, "[[", "shrinkage"
         )
       )
     ),
@@ -219,7 +243,7 @@ test_that("shrinkage is handled correctly", {
       test_data,
       "time",
       "group"
-    )$parameter[58],
+    )$parameter[76],
     "xi"
   )
 })
@@ -246,7 +270,7 @@ test_that("noncentered splines are handled correctly", {
             "time",
             "group",
             debug = debug
-          )$stan$model_vars,
+          )$stan$channel_vars,
           "[[",
           "noncentered"
         )
@@ -274,7 +298,7 @@ test_that("lower bounds for tau are handled correctly", {
         test_data,
         "time", "group",
         debug = debug
-      )$stan$model_vars,
+      )$stan$channel_vars,
       "[[",
       "lb"
     ),
@@ -308,6 +332,7 @@ test_that("latent factors are handled correctly", {
     ]
   )
 })
+
 # Lag edgecases -----------------------------------------------------------
 
 test_that("lags are parsed", {
@@ -480,6 +505,26 @@ test_that("no groups data preparation works", {
   )
 })
 
+test_that("no groups group variable name generation works", {
+  d <- gaussian_example |> dplyr::filter(id == 1)
+  d$.group <- d$x
+  d$x <- NULL
+  expect_error(
+    fit <- dynamite(
+      dformula =
+        obs(y ~ -1 + z + varying(~ .group + lag(y)), family = "gaussian") +
+        random_spec() +
+        splines(df = 20),
+      data = d,
+      time = "time",
+      debug = list(no_compile = TRUE)
+    ),
+    NA
+  )
+  expect_identical(fit$group_var, ".group_")
+  expect_true(all(fit$data[[fit$group_var]] == 1L))
+})
+
 # Deterministic edgecases -------------------------------------------------
 
 test_that("deterministic channels are parsed", {
